class Column:
    def __init__(self, connection):
        """
        Add, drop, modify, or retrieve a column.

        Note:
            Aggregate or use composition while using this
            class in another object. Also, make sure to
            pass the connection object as a parameter.

        Examples:
            >>> self.col = Column('pass the connection obj')

        Args:
            connection (:obj:`Connection`): Connect to the database.

        Attributes:
            self.__con (:obj:`Connection`): Use composition for connection.
        """

        self.__con = connection

    def __str__(self):
        """
        String representation of the Column class.

        Note:
            Use this in a print statement to print the
            documentation of the constructor.

        Returns:
            Returns the constructor documentation as string.
        """

        return self.__init__.__doc__

    def add(self, table, **columns):
        """
        Add a column in a table.

        Note:
            Pass the table name in the first parameter
            and pass the columns and data types in the
            second parameter. Also, pass as many columns
            as you want.

        Examples:
            >>> self.add('tbl', col1='text', col2='integer')

        Args:
            table (str): Add columns to this table.
            **columns (:obj:`kwargs`): Add columns with data types.

        Keyword Args:
            **columns (:obj:`kwargs`): Create the given columns.
                The first part in key='val' represents the column name
                and the next part represents the data type.

        Returns:
            Returns nothing.
        """

        # iterate through **columns and execute the query
        for column, data_type in columns.items():
            # skip the already existing columns
            if column not in self.fetch_names(table):
                # query for adding column(s) in a table
                query = f'''
                    ALTER TABLE {table}
                    ADD COLUMN {column} {data_type}
                '''

                # execute the query
                self.__con.execute(query)

    def add_fk(self, table, column, reference_tbl, reference_col):
        """
        Add a column and set it as a foreign key.

        Note:
            This can only set a new column as a foreign key.
            To avoid problems, first, create other columns
            and then add a foreign key column using this function.

        Examples:
            >>> self.add_fk('tbl', 'fk_col', 'ref_tbl', 'ref_col')

        Args:
            table (str): Add a foreign key column to this table.
            column (str): Add this new column as a foreign key.
            reference_tbl (str): Refer the foreign key in this table.
            reference_col (str): Refer the foreign key in this column.

        Returns:
            Returns nothing.
        """

        # query for adding a foreign key column
        query = f'''
            ALTER TABLE {table}
            ADD COLUMN {column} INTEGER
            REFERENCES {reference_tbl}({reference_col})
        '''

        # execute the query
        self.__con.execute(query)

    def rename(self, table, current_name, new_name):
        """
        Rename a column in a table.

        Note:
            This function only works on non-primary-key
            columns because SQLite currently does not support
            renaming a primary key.

        Examples:
            >>> self.rename('tbl', 'col_name', 'new_col')

        Args:
            table (str): Rename a column from this table.
            current_name (str): Rename this column.
            new_name (str): New name for the column.

        Returns:
            Returns nothing.
        """

        # query for renaming a column
        query = f'''
            ALTER TABLE {table}
            RENAME COLUMN {current_name}
            TO {new_name}
        '''

        # execute the query
        self.__con.execute(query)

    def fetch(self, table, *columns):
        """
        Fetch column(s) from a table.

        Note:
            Pass the table name in the first parameter
            and the column name(s) in the next parameters.

        Examples:
            >>> print(self.fetch('tbl', 'col1', 'col2'))

        Args:
            table (str): Fetch column(s) from this table.
            *columns (:obj:`list`): Fetch these columns from a table.

        Returns:
            Returns the fetch result after executing the query.
        """

        # store column names to be fetched
        columns = ', '.join(columns)

        # query for fetching one or more columns
        query = f'SELECT {columns} FROM {table}'

        # return the fetch result after executing the query.
        return self.__con.fetch(query)

    def fetch_names(self, table):
        """
        Fetch column names from a table.

        Note:
            Pass the table name as a parameter to get the
            list of columns that exists within a table.

        Examples:
            >>> print(self.fetch_names('tbl1'))

        Args:
            table (str): Fetch column names from this table.

        Returns:
            Returns the columns names after executing the query.
        """

        # query for getting the column names
        query = f'''
            PRAGMA table_info({table})
        '''

        # return the column names after executing the query
        return [column['name'] for column in self.__con.fetch(query)]

    def filter(self, table, **col_val):
        """
        Fetch and filter columns with the given values.

        Note:
            Pass the table name in the first parameter and the
            columns along with their values in the other parameters.
            The function return the rows based on the columns
            that have the given values.

        Examples:
            >>> print(self.filter('tbl', col1='val1', col2='val2'))

        Args:
            table (str): Fetch from this table.
            **col_val (:obj:`kwargs`): Fetch based on columns and values


        Keyword Args:
            **col_val (:obj:`kwargs`): The first part in key=val represents
                the column name and the second part represents the value for
                that column.

        Returns:
            Returns the rows and columns after executing the query.
        """

        # store column in (val) conditions for query
        # join the columns along with their values and
        # remove the last extra 'AND' word by using .rsplit(' ', 2)[0]
        col_in_val = str(
            "".join(f"{col} IN ('{val}') AND " for (col, val) in col_val.items())
        ).rsplit(' ', 2)[0]

        # query for fetching rows based on column's value
        query = f'SELECT * FROM {table} WHERE {col_in_val}'

        # execute the query and return the result
        return self.__con.fetch(query)



class Row:
    def __init__(self, connection):
        """
        Insert, delete, modify, or retrieve a row.

        Note:
            Aggregate or use composition while using
            this class in another object. Also, make sure to
            pass the connection object as a parameter.

        Examples:
            >>> self.row = Row('pass connection obj')

        Args:
            connection (:obj:`Connection`): Connect to the database.

        Attributes:
            self.__con (:obj:`Connection`): Use composition for connection.
        """

        self.__con = connection

    def __str__(self):
        """
        String representation of the Row class.

        Note:
            Use this in a print statement to print the
            documentation of the constructor.

        Returns:
            Returns the constructor documentation as string.
        """

        return self.__init__.__doc__

    def insert(self, table, **data):
        """
        Insert data in a table.

        Notes:
            Pass the table name in the first parameter and
            pass a **kwargs type in the second parameter.

        Examples:
            >>> self.insert('tbl', col1='text', col2='integer')

        Args:
            table (str): Insert given data in this table.
            **data (:obj:`kwargs`): Insert this data in the given table.

        Keyword Args:
            **data (:obj:`kwargs`): Add the given data.
                The first part in key='val' represents the column
                name and the next part represents data type.

        Returns:
            Returns nothing.
        """

        # get column names join them using comma
        columns = ', '.join(data.keys())

        # get the values for columns as string
        values = str(list(data.values()))[1:-1]

        # query for inserting data in a table
        query = f'''
           INSERT INTO {table} ({columns})
           VALUES ({values})
        '''

        # execute the query
        self.__con.execute(query)

    def delete(self, table, row_id):
        """
        Delete a row from a table.

        Note:
            Pass the table name in the first parameter and
            the primary key in the second parameter.

        Examples:
            >>> self.delete('tbl', 1)

        Args:
            table (str): Delete row from this table.
            row_id (int): Delete row with this primary key.

        Returns:
            Returns nothing.
        """

        # structure of the primary key
        primary_key = f"{table}_id"

        # query for deleting a row from a table
        query = f'''
           DELETE FROM {table}
           WHERE {primary_key} = {row_id}
        '''

        # execute the query
        self.__con.execute(query)

    def update(self, table, row_id, **data):
        """
        Update a row in a table.

        Note:
            Pass the table name in the first parameter and
            pass the primary key in the second one. Also,
            the datatype for the third parameter is '**kwargs'.
            The first pair in the third parameter represents
            the column name and the second pair represents the
            value for that column.

        Examples:
            >>> self.update('tbl', 1, name='R', age='25')

        Args:
            table (str): Update a row from this table.
            row_id (int): Update a row with this id.
            **data (:obj:`kwargs`): Update a row with this data.

        Keyword Args:
            **data (:obj:`kwargs`): Update with the given data.
                First part in key='val' represents the column name
                and the next part represents data.

        Returns:
            Returns nothing.
        """

        # store columns and their values
        columns_values = "".join(
            f"{col}='{val}'," for (col, val) in data.items()
        )[0:-1]

        # structure for primary key
        primary_key = f"{table}_id"

        # query for updating row(s) in a table
        query = f'''
            UPDATE {table}
            SET {columns_values}
            WHERE {primary_key} = {row_id}
        '''

        # execute the query
        self.__con.execute(query)

    def fetch(self, table, row_id, *columns):
        """
        Fetch specific cells within a row

        Note:
            Pass the table name in the first parameter,
            and the primary key in the second parameter.
            Pass one or more column names in the third
            parameter.

        Examples:
            >>> print(self.fetch('tbl', 1, 'col1', 'col2'))

        Args:
            table (str): Fetch cells from this table.
            row_id (int): Fetch cells with this primary key.
            *columns (str): Fetch cells from these columns.

        Returns:
            Returns the fetch result after executing the query.
        """

        # store column names to be fetched
        columns = ', '.join(columns)

        # structure for the primary key
        primary_key = f'{table}_id'

        # query for fetching one or more cells from a table
        query = f'''
            SELECT {columns}
            FROM {table}
            WHERE {primary_key} = {row_id}
        '''

        # return the fetch result after executing the query
        return self.__con.fetch(query)[0]

    def count(self, table, **col_val):
        """
        Count number of rows for columns with the given values.

        Note:
            Pass the table name in the first parameter and the
            columns along with their values in the other parameters.
            The function counts the number of rows based on the columns
            that have the given values.

        Examples:
            >>> print(self.count('tbl', col1='val1', col2='val2'))

        Args:
            table (str): Count rows from this table.
            **col_val (:obj:`kwargs`): Count rows based on columns and values


        Keyword Args:
            **col_val (:obj:`kwargs`): The first part in key=val represents
                the column name and the second part represents the value for
                that column.

        Returns:
            Returns the number of rows after executing the query.
        """

        # store column in (val) conditions for query
        # join the columns along with their values and
        # remove the last extra 'AND' word by using .rsplit(' ', 2)[0]
        col_in_val = str(
            "".join(f"{col} IN ('{val}') AND " for (col, val) in col_val.items())
        ).rsplit(' ', 2)[0]

        # query for counting number of rows based on a column's value
        query = f'SELECT COUNT(*) FROM {table} WHERE {col_in_val}'

        # return the number of rows after executing the query
        return self.__con.fetch(query)[0]['COUNT(*)']



class Database:
    def __init__(self, connection, table):
        """
        Create a database with a given name and location.

        Note:
            the proper way to use this API is to create
            an instance of the Schema object. However, if you
            want to make an instance of this class, pass the name
            and location. If you want to create the database in your
            working directory, you can leave the second parameter empty.
            Also, if the name is left empty, the database will be
            created with 'main' name.

        Examples:
            >>> self.__db = Database('name', 'directory...')
            >>> self.__db = Database() # or empty parameters

        Args:
            connection (:obj:`Connection`): Create the database using this
                connection.

        Attributes:
            self.__con (:obj:`Connection`): Use composition for Connection.
            self.__tbl (:obj:`Table`): Use composition for the Table class.
        """

        # use composition for Connection class
        self.con = connection

        # use composition for Table class
        self.tbl = table

    def __str__(self):
        """
        String representation of the Database class.

        Note:
            Use this in a print statement to print the
            documentation of the constructor.

        Returns:
            Returns the constructor documentation as string.
        """

        return self.__init__.__doc__

    def drop_col(self, tbl, **columns):
        """
        Drop a column from the database.

        Note:
            Pass the table name in the first parameter and
            name of the columns that you want to keep, along with
            their data types, in the second parameter. SQLite does not
            support dropping a column. To make this work, a temporary
            copy of the table with the desired columns will be created.
            After copying the data, the temporary table will be deleted.

        Examples:
            >>> self.drop_col('tbl', name='text', age='integer')

        Args:
            tbl (str): Drop a column from this table.
            **columns (:obj:`kwargs`): Keep these columns and
                drop the ones that are not mentioned.

        Keyword Args:
            **columns (:obj:`kwargs`): Keep the given columns.
                The first part in key='val' represents the column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        # name of the temporary table
        temp = "temp"

        # copy the old table in the temp table
        self.tbl.copy(tbl, temp, **columns)

        # drop the old table
        self.tbl.drop(tbl)

        # copy the temp table in a new table
        # the new table has the same name as the old one
        self.tbl.copy(temp, tbl, **columns)

        # drop the temp table
        self.tbl.drop(temp)



class Table:
    def __init__(self, connection, row, column):
        """
        Add, drop, copy, or modify a table.

        Note:
            Use aggregation or composition for using this
            class in another object. The reason is that
            table has a 'has a' relationship with columns and rows.
            Make sure to pass the connection object as its parameter.

        Examples:
            >>> self.tbl = Table('pass connection obj')

        Args:
            connection (:obj:`Connection`): Connect to the file.

        Attributes:
              self.__con (:obj:`Connection`): Connect using composition.
              self.col(:obj:`Column`): Use composition for Column class.
              self.row(:obj:`Row`): Use composition for Row class.
        """

        # use composition for connection
        self.__con = connection

        # Use composition for column and row
        self.col = column
        self.row = row

    def __str__(self):
        """
        String representation of the Table class.

        Note:
            Use this in a print statement to print the
            documentation of the constructor.

        Returns:
            Returns the constructor documentation as string.
        """

        return self.__init__.__doc__

    def add(self, *tables):
        """
        Add a table in the database.

        Note:
            Add one or many tables at once. Make sure to separate
            their names while passing as a parameter.

        Examples:
            >>> self.add('tbl1', 'tbl2', 'tbl3')

        Args:
            *tables (:obj:`list`): Add the given tables.

        Returns: Returns nothing.
        """

        # iterate through **tables and execute the query
        for table in tables:
            # structure of the primary key
            primary_key = f"{table}_id"

            # query for creating table(s)
            query = f'''
                CREATE TABLE IF NOT EXISTS {table} (
                {primary_key} INTEGER PRIMARY KEY AUTOINCREMENT
            )'''

            # execute the query
            self.__con.execute(query)

    def drop(self, *tables):
        """
        Drop a table from the database.

        Note:
            Pass one or many table names as parameters to drop them.

        Examples:
            >>> self.drop('tbl1', 'tbl2', 'tbl3')

        Args:
            *tables (:obj:`list`): Drop one or many tables.

        Returns:
            Returns nothing.
        """

        # iterate through **tables parameter and execute the query
        for table in tables:
            # query for dropping a table
            query = f"DROP TABLE IF EXISTS {table}"

            # execute the query
            self.__con.execute(query)

    def rename(self, old_name, new_name):
        """
        Rename an existing table in the database.

        Note:
            Pass the name of an existing table in the
            first parameter and the new name in the
            second parameter.

        Examples:
            >>> self.rename('old_name', 'new_name')

        Args:
            old_name (str): Rename this table.
            new_name (str): New name for the given table.

        Returns:
            Returns nothing.
        """

        # query for renaming a table
        query = f"ALTER TABLE {old_name} RENAME TO {new_name}"

        # execute the query
        self.__con.execute(query)

    def form(self, table, **columns):
        """
        Create a table with the given column names and data types.

        Note:
            Pass the table name in the first parameter and the column
            names along with their data types in the second parameter.
            If you want to add a foreign key column, do not include
            the column name here. Instead, create the table with normal
            columns and then use 'add_fk()' function from the Column
            class to add a new column with a foreign key constraint.

        Examples:
            >>> self.form('tbl', col1='text', col2='integer')

        Args:
            table (str): Create this table.
            **columns (:obj:`kwargs`): Add these columns to the table.

        Keyword Args:
            **columns (:obj:`kwargs`): Form table using the given
                columns. The first part in key='val' represents column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        # create the table
        self.add(table)

        # add columns to the table
        self.col.add(table, **columns)

    def copy(self, origin_tbl, new_tbl, **columns):
        """
        Copy an existing table into a new one.

        Note:
            Pass the name of the original table in the first
            parameter and the name of the new table in the second
            parameter. Also, pass the name of the columns that you want
            to copy along with their data types in the third parameter.

        Examples:
            >>> self.copy('tbl', 'new_tbl', col1='text', col2='integer')

        Args:
            origin_tbl (str): Copy this table.
            new_tbl (str): Copy the old table in this table.
            **columns (:obj:`kwargs`): Keep these columns.

        Keyword Args:
            **columns (:obj:`kwargs`): Keep the given columns.
                The first part in key='val' represents the column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        # store the column names by joining them using comma
        col = ', '.join(columns.keys())

        # form the table using given columns
        self.form(new_tbl, **columns)

        # query for inserting selected column from one table to another
        query = f'''
            INSERT INTO {new_tbl}({col}) SELECT {col} FROM {origin_tbl}
        '''

        # execute the query
        self.__con.execute(query)

    def fetch(self, table):
        """
        Fetch a table from the database.

        Note:
            Pass the table name as a parameter
        
        Examples:
            >>> print(self.fetch('table_name'))
        
        Args:
            table (str): Fetch data from this table.

        Returns:
            Returns the fetch result after executing the query.
        """

        # query for getting data from a table
        query = f'SELECT * FROM {table}'

        # return the fetch result after executing the query
        return self.__con.fetch(query)



import sqlite3
from .file import File


class Connection:
    def __init__(self, name=None, location=None):
        """
        Connect to the database file using a given name and location.

        Note:
            Leave the parameters empty if you want to connect to a file
            in the working directory. The default for the name is 'main'.
            If an SQLite '.db' file in the desired location is not present,
            it will create it.

        Examples:
            >>> self.conn = Connection('name', 'location')

        Args:
            name (str, optional): Connect to the file with this name.
                If the name is empty, it will connect to the 'main' file.

            location (str, optional): Connect to the file in this
                location. If location is empty, it will connect to a file
                in the working directory.

        Attributes:
            self.db_file (:obj:`str`): Store full directory address of the file.
        """

        # full directory address of the file
        self.__db_file = str(File(name, location))

        # create/connect to the database file
        self.execute("")

    def __str__(self):
        """
        String representation of the Connection class.

        Note:
            Use this in a print statement to print the
            documentation of the constructor.

        Returns:
            Returns the constructor documentation as string.
        """

        return self.__init__.__doc__

    def execute(self, query, fetch=False):
        """
        Connect to the '.db' file and execute a query.

        Note:
            Pass the query to execute it. If parameters are empty,
            it will only connect to the file.

        Examples:
            >>> self.execute("")
            >>> self.execute('select * from tbl')

        Args:
            fetch (bool, optional): Return the fetch result, if values
                is passed as true. Otherwise, it returns nothing.
            query (str): Execute the given query.

        Returns:
            Returns the fetch result after executing a query.
            To return the fetch result, the second parameter
            must be set to True.
        """

        # use context manager for connection
        with sqlite3.connect(self.__db_file) as con:

            # to return the fetch result as dictionary
            def dict_factory(cur, row):
                d = {}
                for idx, col in enumerate(cur.description):
                    d[col[0]] = row[idx]
                return d

            # use dict_factory for row_factory
            # it will return dictionaries for fetch results
            con.row_factory = dict_factory

            # get the cursor from the connection
            cursor = con.cursor()

            # if query is given, execute it
            # otherwise, only connect to the file
            cursor.execute(query)

            # return the fetch result if asked for
            return cursor.fetchall() if fetch else None

    def fetch(self, query):
        """
        Execute a query and return the fetch result.

        Note:
            Use this function if you want the return
            value. Otherwise, use the execute() function.

        Examples:
            >>> print(self.fetch('select * from tbl'))

        Args:
            query (str): Fetch this query and return the result.

        Returns:
            Returns the fetch result after executing a query
        """
        return self.execute(query, fetch=True)



import os


class File:
    def __init__(self, name=None, location=None):
        """
        Create and store information about name and location of file.

        Note:
            Do not pass the file name and location as parameters if
            you want to create a 'main.db' file in your directory.

        Examples:
            >>> file = File('name', 'directory')

        Args:
            name (str, optional): Get file name.
            location (str, optional): Get file location.

        Attributes:
            self.name (str): Store file name and if none is given,
                set it as 'main'. Also, it cannot be accessed from outside.

            self.location (str): Store file directory and if none is given,
                set to the current directory. Also, cannot be accessed from
                the outside of the class.

            self.__full_directory (str): Merge file location and name to create
                a full directory address of the SQLite file.
        """

        # store the name of the file, default to 'main'
        self.__name = name or 'main'

        # store the location of the file, default to current directory
        self.__location = location or self.current_directory()

        # store full directory address of the file
        self.__full_directory = f"{self.__location}/{self.__name}.db"

    def __str__(self):
        """
        String representation of the File class.

        Note:
            Use the class directly in a print statement or aggregate
            to get the full directory of the file as a string.

        Returns:
            Returns the full directory as a string.
        """

        return self.__full_directory

    @staticmethod
    def current_directory():
        """
        Get current project directory.

        Note:
            For windows users, it will change '\' to '/' automatically
            because python refers to directories using forward slash.

        Returns:
            :return: Returns the current working directory.
        """

        return str(os.getcwd().replace('\\', '/'))



from hydra.schema import Schema


"""
Copyright 2021 Rahmat Azad

Permission is hereby granted, free of charge,
to any person obtaining a copy of this software
and associated documentation files (the "Software"),
to deal in the Software without restriction,
including without limitation the rights to use, copy,
modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this
permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
THE AUTHOR OR COPYRIGHT HOLDER BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""


from .column import Column
from .connection import Connection
from .database import Database
from .row import Row
from .table import Table


class Schema:
    def __init__(self, name=None, location=None):
        """
        Create a schema and provide easy access to operations.

        Note:
            Pass the file name in the first parameter and the
            file location in the second parameter. If name or location
            is not provided, it will apply the default values. This object
            is another layer of abstraction that uses the facade pattern to
            provide an easy access to the operations of each object.

        Examples:
            >>> schema = Schema('name', 'location') # with parameters
            >>> db = Schema() # empty parameter
            >>> db.add_table('tbl1', 'tbl2') # access to operations

        Args:
            name (str, optional): Create the database with this name.
                Defaults to 'main'.
            location (str, optional): Create the database in this location.
                Defaults to the current working directory.

        Attributes:
            self.__con(:obj:`Connection`): Aggregate Connection object.
            self.__db(:obj:`Database`): Aggregate Database object.
            self.__tbl(:obj:`Table`): Aggregate Table object.
            self.__col(:obj:`Column`): Aggregate Column object.
            self.__row(:obj:`Row`): Aggregate Row object.
        """

        # aggregate the Connection object
        self.__con = Connection(name, location)

        # aggregate the Row object
        self.__row = Row(self.__con)

        # aggregate the Column object
        self.__col = Column(self.__con)

        # aggregate the Table object
        self.__tbl = Table(self.__con, self.__row, self.__col)

        # aggregate the database object
        self.__db = Database(self.__con, self.__tbl)

    def execute(self, query):
        """
        Connect to the '.db' file and execute a query.

        Note:
            Pass the query to execute it. If parameters are empty,
            it will only connect to the file.

        Examples:
            >>> self.execute("")
            >>> self.execute('create table tbl_name')

        Args:
            query (str): Execute the given query.

        Returns:
            Returns nothing.
        """

        self.__con.execute(query)

    def fetch(self, query):
        """
        Execute a query and return the fetch result.

        Note:
            Use this function if you want the return
            value. Otherwise, use the execute() function.

        Examples:
            >>> print(self.fetch('select * from tbl'))

        Args:
            query (str): Fetch this query and return the result.

        Returns:
            Returns the fetch result after executing a query
        """

        return self.__con.fetch(query)

    def add_table(self, *tables):
        """
        Add a table in the database.

        Note:
            Add one or many tables at once. Make sure to separate
            their names while passing as a parameter.

        Examples:
            >>> self.add_table('tbl1', 'tbl2', 'tbl3')

        Args:
            *tables (:obj:`list`): Add the given tables.

        Returns: Returns nothing.
        """

        self.__tbl.add(*tables)

    def drop_table(self, *tables):
        """
        Drop a table from the database.

        Note:
            Pass one or many table names as parameters to drop them.

        Examples:
            >>> self.drop_table('tbl1', 'tbl2', 'tbl3')

        Args:
            *tables (:obj:`list`): Drop one or many tables.

        Returns:
            Returns nothing.
        """

        self.__tbl.drop(*tables)

    def rename_table(self, old_name, new_name):
        """
        Rename an existing table in the database.

        Note:
            Pass the name of an existing table in the
            first parameter and the new name in the
            second parameter.

        Examples:
            >>> self.rename_table('old_name', 'new_name')

        Args:
            old_name (str): Rename this table.
            new_name (str): New name for the given table.

        Returns:
            Returns nothing.
        """

        self.__tbl.rename(old_name, new_name)

    def form_table(self, table, **columns):
        """
        Create a table with the given column names and data types.

        Note:
            Pass the table name in the first parameter and the column
            names along with their data types in the second parameter.
            If you want to add a foreign key column, do not include
            the column name here. Instead, create the table with normal
            columns and then use 'add_fk()' function from the Column
            class to add a new column with a foreign key constraint.

        Examples:
            >>> self.form_table('tbl', col1='text', col2='integer')

        Args:
            table (str): Create this table.
            **columns (:obj:`kwargs`): Add these columns to the table.

        Keyword Args:
            **columns (:obj:`kwargs`): Form table using the given
                columns. The first part in key='val' represents column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        self.__tbl.form(table, **columns)

    def copy_table(self, origin_tbl, new_tbl, **columns):
        """
        Copy an existing table into a new one.

        Note:
            Pass the name of the original table in the first
            parameter and the name of the new table in the second
            parameter. Also, pass the name of the columns that you want
            to copy along with their data types in the third parameter.

        Examples:
            >>> self.copy_table('tbl', 'new_tbl', col1='text', col2='integer')

        Args:
            origin_tbl (str): Copy this table.
            new_tbl (str): Copy the old table in this table.
            **columns (:obj:`kwargs`):

        Keyword Args:
            **columns (:obj:`kwargs`): Keep the given columns.
                The first part in key='val' represents the column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        self.__tbl.copy(origin_tbl, new_tbl, **columns)

    def fetch_table(self, table):
        """
        Fetch a table from the database.

        Note:
            Pass the table name as a parameter.

        Examples:
            >>> print(self.fetch_table('table_name'))

        Args:
            table (str): Fetch data from this table.

        Returns:
            Returns the fetch result after executing the query.
        """

        return self.__tbl.fetch(table)

    def add_column(self, table, **columns):
        """
        Add a column in a table.

        Note:
            Pass the table name in the first parameter
            and pass the columns and data types in the
            second parameter. Also, pass as many columns
            as you want.

        Examples:
            >>> self.add_column('tbl', col1='text', col2='integer')

        Args:
            table (str): Add columns to this table.
            **columns (:obj:`kwargs`): Add columns with data types.

        Keyword Args:
            **columns (:obj:`kwargs`): Create the given columns.
                The first part in key='val' represents the column name
                and the next part represents the data type.

        Returns:
            Returns nothing.
        """

        self.__col.add(table, **columns)

    def drop_column(self, table, **columns):
        """
        Drop a column from the database.

        Note:
            Pass the table name in the first parameter and
            name of the columns that you want to keep, along with
            their data types, in the second parameter. SQLite does not
            support dropping a column. To make this work, a temporary
            copy of the table with the desired columns will be created.
            After copying the data, the temporary table will be deleted.

        Examples:
            >>> self.drop_column('tbl', name='text', age='integer')

        Args:
            table (str): Drop a column from this table.
            **columns (:obj:`kwargs`): Keep these columns and
                drop the ones that are not mentioned.

        Keyword Args:
            **columns (:obj:`kwargs`): Keep the given columns.
                The first part in key='val' represents the column name
                and the next part represents data type.

        Returns:
            Returns nothing.
        """

        self.__db.drop_col(table, **columns)

    def add_fk(self, table, column, reference_tbl, reference_col):
        """
        Add a column and set it as a foreign key.

        Note:
            This can only set a new column as a foreign key.
            To avoid problems, first, create other columns
            and then add a foreign key column using this function.

        Examples:
            >>> self.add_fk('tbl', 'fk_col', 'ref_tbl', 'ref_col')

        Args:
            table (str): Add a foreign key column to this table.
            column (str): Add this new column as a foreign key.
            reference_tbl (str): Refer the foreign key in this table.
            reference_col (str): Refer the foreign key in this column.

        Returns:
            Returns nothing.
        """

        self.__col.add_fk(table, column, reference_tbl, reference_col)

    def rename_column(self, tbl, current_name, new_name):
        """
        Rename a column in a table.

        Note:
            This function only works on non-primary-key
            columns because SQLite currently does not support
            renaming a primary key.

        Examples:
            >>> self.rename_column('tbl', 'col_name', 'new_col')

        Args:
            tbl (str): Rename a column from this table.
            current_name (str): Rename this column.
            new_name (str): New name for the column.

        Returns:
            Returns nothing.
        """

        self.__col.rename(tbl, current_name, new_name)

    def fetch_columns(self, table, *columns):
        """
        Fetch column(s) from a table.

        Note:
            Pass the table name in the first parameter
            and the column name(s) in the next parameters.

        Examples:
            >>> print(self.fetch_columns('tbl', 'col1', 'col2'))

        Args:
            table (str): Fetch column(s) from this table.
            *columns (:obj:`list`): Fetch these columns from a table.

        Returns:
            Returns the fetch result after executing the query.
        """

        return self.__col.fetch(table, *columns)

    def fetch_column_names(self, table):
        """
        Fetch column names from a table.

        Note:
            Pass the table name as a parameter to get the
            list of columns that exists within a table.

        Examples:
            >>> print(self.fetch_column_names('tbl1'))

        Args:
            table (str): Fetch column names from this table.

        Returns:
            Returns the columns names after executing the query.
        """

        return self.__col.fetch_names(table)

    def filter_column(self, table, **col_val):
        """
        Fetch and filter columns with the given values.

        Note:
            Pass the table name in the first parameter and the
            columns along with their values in the other parameters.
            The function return the rows based on the columns
            that have the given values.

        Examples:
            >>> print(self.filter_column('tbl', col1='val1', col2='val2'))

        Args:
            table (str): Fetch from this table.
            **col_val (:obj:`kwargs`): Fetch based on columns and values


        Keyword Args:
            **col_val (:obj:`kwargs`): The first part in key=val represents
                the column name and the second part represents the value for
                that column.

        Returns:
            Returns the rows and columns after executing the query.
        """

        return self.__col.filter(table, **col_val)

    def insert_row(self, table, **data):
        """
        Insert data in a table.

        Notes:
            Pass the table name in the first parameter and
            pass a **kwargs type in the second parameter.

        Examples:
            >>> self.insert_row('tbl', col1='text', col2='integer')

        Args:
            table (str): Insert given data in this table.
            **data (:obj:`kwargs`): Insert this data in the given table.

        Keyword Args:
            **data (:obj:`kwargs`): Add the given data.
                The first part in key='val' represents the column
                name and the next part represents data type.

        Returns:
            Returns nothing.
        """

        self.__row.insert(table, **data)

    def delete_row(self, table, row_id):
        """
        Delete a row from a table.

        Note:
            Pass the table name in the first parameter and
            the primary key in the second parameter.

        Examples:
            >>> self.delete_row('tbl', 1)

        Args:
            table (str): Delete row from this table.
            row_id (int): Delete row with this primary key.

        Returns:
            Returns nothing.
        """

        self.__row.delete(table, row_id)

    def update_row(self, table, row_id, **data):
        """
        Update a row in a table.

        Note:
            Pass the table name in the first parameter and
            pass the primary key in the second one. Also,
            the datatype for the third parameter is '**kwargs'.
            The first pair in the third parameter represents
            the column name and the second pair represents the
            value for that column.

        Examples:
            >>> self.update_row('tbl', 1, name='R', age='25')

        Args:
            table (str): Update a row from this table.
            row_id (int): Update a row with this id.
            **data (:obj:`kwargs`): Update a row with this data.

        Keyword Args:
            **data (:obj:`kwargs`): Update with the given data.
                First part in key='val' represents the column name
                and the next part represents data.

        Returns:
            Returns nothing.
        """

        self.__row.update(table, row_id, **data)

    def fetch_row(self, table, row_id, *columns):
        """
        Fetch specific cells within a row

        Note:
            Pass the table name in the first parameter,
            and the primary key in the second parameter.
            Pass one or more column names in the third
            parameter.

        Examples:
            >>> print(self.fetch_row('tbl', 1, 'col1', 'col2'))

        Args:
            table (str): Fetch cells from this table.
            row_id (int): Fetch cells with this primary key.
            *columns (str): Fetch cells from these columns.

        Returns:
            Returns the fetch result after executing the query.
        """

        # return the fetch result after executing the query
        return self.__row.fetch(table, row_id, *columns)

    def count_rows(self, table, **col_val):
        """
        Count number of rows for columns with the given values.

        Note:
            Pass the table name in the first parameter and the
            columns along with their values in the other parameters.
            The function counts the number of rows based on the columns
            that have the given values.

        Examples:
            >>> print(self.count_rows('tbl', col1='val1', col2='val2'))

        Args:
            table (str): Count rows from this table.
            **col_val (:obj:`kwargs`): Count rows based on columns and values


        Keyword Args:
            **col_val (:obj:`kwargs`): The first part in key=val represents
                the column name and the second part represents the value for
                that column.

        Returns:
            Returns the number of rows after executing the query.
        """

        # return the number of rows after executing the query
        return self.__row.count(table, **col_val)



from setuptools import setup

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

with open("README.txt", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='hydradb',
    version='3.0.0',
    packages=['hydra'],
    url='https://github.com/rhmtazad/hydraDB',
    license='MIT License',
    classifiers=classifiers,
    author='Rahmat Azad',
    author_email='rhmtazad@gmail.com',
    description='An API for faster and easier SQLite/Python operations',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['']
)
